@@ -2,8 +2,8 @@
 
 import abc
 import configparser
-import datetime
-from typing import Any, Mapping, Optional
+from datetime import datetime
+from typing import Any, Mapping, Optional, Type, TypeVar
 
 from autosuspend.util import logger_by_class_instance
 
@@ -15,7 +15,8 @@ class ConfigurationError(RuntimeError):
 
 
 class TemporaryCheckError(RuntimeError):
-    """Indicates a temporary error while performing a check.
+    """
+    Indicates a temporary error while performing a check.
 
     Such an error can be ignored for some time since it might recover
     automatically.
@@ -25,14 +26,18 @@ class TemporaryCheckError(RuntimeError):
 
 
 class SevereCheckError(RuntimeError):
-    """Indicates a sever check error that will probably not recover.
+    """
+    Indicates a sever check error that will probably not recover.
 
-    There no hope this situation recovers.
+    There is no hope this situation recovers.
     """
 
     pass
 
 
+CheckType = TypeVar("CheckType", bound="Check")
+
+
 class Check(abc.ABC):
     """Base class for all kinds of checks.
 
@@ -45,7 +50,9 @@ class Check(abc.ABC):
 
     @classmethod
     @abc.abstractmethod
-    def create(cls, name: str, config: configparser.SectionProxy) -> "Check":
+    def create(
+        cls: Type[CheckType], name: str, config: configparser.SectionProxy
+    ) -> CheckType:
         """Create a new check instance from the provided configuration.
 
         Args:
@@ -61,7 +68,7 @@ class Check(abc.ABC):
         """
         pass
 
-    def __init__(self, name: str = None) -> None:
+    def __init__(self, name: Optional[str] = None) -> None:
         if name:
             self.name = name
         else:
@@ -114,7 +121,7 @@ class Wakeup(Check):
     """Represents a check for potential wake up points."""
 
     @abc.abstractmethod
-    def check(self, timestamp: datetime.datetime) -> Optional[datetime.datetime]:
+    def check(self, timestamp: datetime) -> Optional[datetime]:
         """Indicate if a wakeup has to be scheduled for this check.
 
         Args:
