@@ -1,29 +1,46 @@
 import configparser
+from contextlib import suppress
 import copy
 from datetime import datetime, timedelta, timezone
-import glob
 from io import BytesIO
 import json
 import os
-import pwd
+from pathlib import Path
 import re
 import socket
 import subprocess
+from textwrap import shorten
 import time
-from typing import Any, Dict, Iterable, Optional, Pattern, Sequence, Tuple
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Pattern,
+    Tuple,
+    TYPE_CHECKING,
+)
 import warnings
 
 import psutil
 
 from . import Activity, Check, ConfigurationError, SevereCheckError, TemporaryCheckError
 from .util import CommandMixin, NetworkMixin, XPathMixin
-from ..util.systemd import list_logind_sessions
+from ..util.subprocess import raise_severe_if_command_not_found
+from ..util.systemd import list_logind_sessions, LogindDBusException
+from ..util.xorg import list_sessions_logind, list_sessions_sockets, XorgSession
+
+
+if TYPE_CHECKING:
+    from jsonpath_ng import JSONPath
 
 
 class ActiveCalendarEvent(NetworkMixin, Activity):
     """Determines activity by checking against events in an icalendar file."""
 
-    def __init__(self, name: str, **kwargs) -> None:
+    def __init__(self, name: str, **kwargs: Any) -> None:
         NetworkMixin.__init__(self, **kwargs)
         Activity.__init__(self, name)
 
@@ -51,7 +68,9 @@ class ActiveConnection(Activity):
 
     @classmethod
     def create(
-        cls, name: str, config: configparser.SectionProxy,
+        cls,
+        name: str,
+        config: configparser.SectionProxy,
     ) -> "ActiveConnection":
         try:
             split_ports = config["ports"].split(",")
@@ -66,19 +85,35 @@ class ActiveConnection(Activity):
         Activity.__init__(self, name)
         self._ports = ports
 
+    def normalize_address(
+        self, family: socket.AddressFamily, address: str
+    ) -> Tuple[socket.AddressFamily, str]:
+        if family == socket.AF_INET6:
+            # strip scope
+            return family, address.split("%")[0]
+        elif family == socket.AF_INET:
+            # convert to IPv6 to handle cases where an IPv4 address is targeted via IPv6
+            # to IPv4 mapping
+            return socket.AF_INET6, f"::ffff:{address}"
+        else:
+            return family, address
+
     def check(self) -> Optional[str]:
+        # Find the addresses of the system
         own_addresses = [
-            (item.family, item.address.split("%")[0])
+            self.normalize_address(item.family, item.address)
             for sublist in psutil.net_if_addrs().values()
             for item in sublist
         ]
+        # Find established connections to target ports
         connected = [
-            c.laddr[1]
-            for c in psutil.net_connections()
+            connection.laddr[1]
+            for connection in psutil.net_connections()
             if (
-                (c.family, c.laddr[0]) in own_addresses
-                and c.status == "ESTABLISHED"
-                and c.laddr[1] in self._ports
+                self.normalize_address(connection.family, connection.laddr[0])
+                in own_addresses
+                and connection.status == "ESTABLISHED"
+                and connection.laddr[1] in self._ports
             )
         ]
         if connected:
@@ -96,7 +131,8 @@ class ExternalCommand(CommandMixin, Activity):
         try:
             subprocess.check_call(self._command, shell=True)  # noqa: S602
             return "Command {} succeeded".format(self._command)
-        except subprocess.CalledProcessError:
+        except subprocess.CalledProcessError as error:
+            raise_severe_if_command_not_found(error)
             return None
 
 
@@ -123,7 +159,7 @@ class Kodi(NetworkMixin, Activity):
         return cls(name, **cls.collect_init_args(config))
 
     def __init__(
-        self, name: str, url: str, suspend_while_paused: bool = False, **kwargs
+        self, name: str, url: str, suspend_while_paused: bool = False, **kwargs: Any
     ) -> None:
         self._suspend_while_paused = suspend_while_paused
         if self._suspend_while_paused:
@@ -140,18 +176,20 @@ class Kodi(NetworkMixin, Activity):
         NetworkMixin.__init__(self, url=request, **kwargs)
         Activity.__init__(self, name)
 
-    def check(self) -> Optional[str]:
+    def _safe_request_result(self) -> Dict:
         try:
-            reply = self.request().json()
-            if self._suspend_while_paused:
-                if reply["result"]["Player.Playing"]:
-                    return "Kodi actively playing media"
-            else:
-                if reply["result"]:
-                    return "Kodi currently playing"
-            return None
+            return self.request().json()["result"]
         except (KeyError, TypeError, json.JSONDecodeError) as error:
-            raise TemporaryCheckError(error) from error
+            raise TemporaryCheckError("Unable to get or parse Kodi state") from error
+
+    def check(self) -> Optional[str]:
+        reply = self._safe_request_result()
+        if self._suspend_while_paused:
+            return (
+                "Kodi actively playing media" if reply.get("Player.Playing") else None
+            )
+        else:
+            return "Kodi currently playing" if reply else None
 
 
 class KodiIdleTime(NetworkMixin, Activity):
@@ -169,7 +207,7 @@ class KodiIdleTime(NetworkMixin, Activity):
     def create(cls, name: str, config: configparser.SectionProxy) -> "KodiIdleTime":
         return cls(name, **cls.collect_init_args(config))
 
-    def __init__(self, name: str, url: str, idle_time: int, **kwargs) -> None:
+    def __init__(self, name: str, url: str, idle_time: int, **kwargs: Any) -> None:
         request = url + (
             '?request={{"jsonrpc": "2.0", "id": 1, '
             '"method": "XBMC.GetInfoBooleans",'
@@ -187,7 +225,7 @@ class KodiIdleTime(NetworkMixin, Activity):
             else:
                 return None
         except (KeyError, TypeError, json.JSONDecodeError) as error:
-            raise TemporaryCheckError(error) from error
+            raise TemporaryCheckError("Unable to get or parse Kodi state") from error
 
 
 class Load(Activity):
@@ -244,32 +282,45 @@ class Mpd(Activity):
         return state
 
     def check(self) -> Optional[str]:
+        from mpd import MPDError
+
         try:
             state = self._get_state()
             if state["state"] == "play":
                 return "MPD currently playing"
             else:
                 return None
-        except (ConnectionError, socket.timeout, socket.gaierror) as error:
-            raise TemporaryCheckError(error) from error
+        except (MPDError, ConnectionError, socket.timeout, socket.gaierror) as error:
+            raise TemporaryCheckError("Unable to get the current MPD state") from error
 
 
 class NetworkBandwidth(Activity):
+    @classmethod
+    def _ensure_interfaces_exist(cls, interfaces: Iterable[str]) -> None:
+        host_interfaces = psutil.net_if_addrs().keys()
+        for interface in interfaces:
+            if interface not in host_interfaces:
+                raise ConfigurationError(
+                    "Network interface {} does not exist".format(interface)
+                )
+
+    @classmethod
+    def _extract_interfaces(cls, config: configparser.SectionProxy) -> List[str]:
+        interfaces = config["interfaces"].split(",")
+        interfaces = [i.strip() for i in interfaces if i.strip()]
+        if not interfaces:
+            raise ConfigurationError("No interfaces configured")
+        cls._ensure_interfaces_exist(interfaces)
+        return interfaces
+
     @classmethod
     def create(
-        cls, name: str, config: configparser.SectionProxy,
+        cls,
+        name: str,
+        config: configparser.SectionProxy,
     ) -> "NetworkBandwidth":
         try:
-            interfaces = config["interfaces"].split(",")
-            interfaces = [i.strip() for i in interfaces if i.strip()]
-            if not interfaces:
-                raise ConfigurationError("No interfaces configured")
-            host_interfaces = psutil.net_if_addrs().keys()
-            for interface in interfaces:
-                if interface not in host_interfaces:
-                    raise ConfigurationError(
-                        "Network interface {} does not exist".format(interface)
-                    )
+            interfaces = cls._extract_interfaces(config)
             threshold_send = config.getfloat("threshold_send", fallback=100)
             threshold_receive = config.getfloat("threshold_receive", fallback=100)
             return cls(name, interfaces, threshold_send, threshold_receive)
@@ -296,6 +347,42 @@ class NetworkBandwidth(Activity):
         self._previous_values = psutil.net_io_counters(pernic=True)
         self._previous_time = time.time()
 
+    @classmethod
+    def _rate(cls, new: float, old: float, new_time: float, old_time: float) -> float:
+        delta = new - old
+        return delta / (new_time - old_time)
+
+    class _InterfaceActive(RuntimeError):
+        pass
+
+    def _check_interface(
+        self,
+        interface: str,
+        new: psutil._common.snetio,
+        old: psutil._common.snetio,
+        new_time: float,
+        old_time: float,
+    ) -> None:
+        # send direction
+        rate_send = self._rate(new.bytes_sent, old.bytes_sent, new_time, old_time)
+        if rate_send > self._threshold_send:
+            raise self._InterfaceActive(
+                "Interface {} sending rate {} byte/s "
+                "higher than threshold {}".format(
+                    interface, rate_send, self._threshold_send
+                )
+            )
+
+        # receive direction
+        rate_receive = self._rate(new.bytes_recv, old.bytes_recv, new_time, old_time)
+        if rate_receive > self._threshold_receive:
+            raise self._InterfaceActive(
+                "Interface {} receive rate {} byte/s "
+                "higher than threshold {}".format(
+                    interface, rate_receive, self._threshold_receive
+                )
+            )
+
     def check(self) -> Optional[str]:
         # acquire the previous state and preserve it
         old_values = self._previous_values
@@ -305,7 +392,7 @@ class NetworkBandwidth(Activity):
         new_values = psutil.net_io_counters(pernic=True)
         self._previous_values = new_values
         new_time = time.time()
-        if new_time == self._previous_time:
+        if new_time <= self._previous_time:
             raise TemporaryCheckError("Called too fast, no time between calls")
         self._previous_time = new_time
 
@@ -313,31 +400,16 @@ class NetworkBandwidth(Activity):
             if interface not in new_values or interface not in self._previous_values:
                 raise TemporaryCheckError("Interface {} is missing".format(interface))
 
-            # send direction
-            delta_send = (
-                new_values[interface].bytes_sent - old_values[interface].bytes_sent
-            )
-            rate_send = delta_send / (new_time - old_time)
-            if rate_send > self._threshold_send:
-                return (
-                    "Interface {} sending rate {} byte/s "
-                    "higher than threshold {}".format(
-                        interface, rate_send, self._threshold_send
-                    )
-                )
-
-            # receive direction
-            delta_receive = (
-                new_values[interface].bytes_recv - old_values[interface].bytes_recv
-            )
-            rate_receive = delta_receive / (new_time - old_time)
-            if rate_receive > self._threshold_receive:
-                return (
-                    "Interface {} receive rate {} byte/s "
-                    "higher than threshold {}".format(
-                        interface, rate_receive, self._threshold_receive
-                    )
+            try:
+                self._check_interface(
+                    interface,
+                    new_values[interface],
+                    old_values[interface],
+                    new_time,
+                    old_time,
                 )
+            except self._InterfaceActive as e:
+                return str(e)
 
         return None
 
@@ -361,17 +433,22 @@ class Ping(Activity):
         self._hosts = hosts
 
     def check(self) -> Optional[str]:
-        for host in self._hosts:
-            cmd = ["ping", "-q", "-c", "1", host]
-            if (
-                subprocess.call(  # noqa: S603 we know the input from the config
-                    cmd, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL,
-                )
-                == 0
-            ):
-                self.logger.debug("host " + host + " appears to be up")
-                return "Host {} is up".format(host)
-        return None
+        try:
+            for host in self._hosts:
+                cmd = ["ping", "-q", "-c", "1", host]
+                if (
+                    subprocess.call(  # noqa: S603 we know the input from the config
+                        cmd,
+                        stdout=subprocess.DEVNULL,
+                        stderr=subprocess.DEVNULL,
+                    )
+                    == 0
+                ):
+                    self.logger.debug("host " + host + " appears to be up")
+                    return "Host {} is up".format(host)
+            return None
+        except FileNotFoundError as error:
+            raise SevereCheckError("Binary ping cannot be found") from error
 
 
 class Processes(Activity):
@@ -390,13 +467,10 @@ class Processes(Activity):
 
     def check(self) -> Optional[str]:
         for proc in psutil.process_iter():
-            try:
+            with suppress(psutil.NoSuchProcess):
                 pinfo = proc.name()
-                for name in self._processes:
-                    if pinfo == name:
-                        return "Process {} is running".format(name)
-            except psutil.NoSuchProcess:
-                pass
+                if pinfo in self._processes:
+                    return "Process {} is running".format(pinfo)
         return None
 
 
@@ -405,13 +479,18 @@ class Smb(Activity):
     def create(cls, name: str, config: Optional[configparser.SectionProxy]) -> "Smb":
         return cls(name)
 
-    def check(self) -> Optional[str]:
+    def _safe_get_status(self) -> str:
         try:
-            status_output = subprocess.check_output(  # noqa: S603, S607
+            return subprocess.check_output(  # noqa: S603, S607
                 ["smbstatus", "-b"]
             ).decode("utf-8")
+        except FileNotFoundError as error:
+            raise SevereCheckError("smbstatus binary not found") from error
         except subprocess.CalledProcessError as error:
-            raise SevereCheckError(error) from error
+            raise TemporaryCheckError("Unable to execute smbstatus") from error
+
+    def check(self) -> Optional[str]:
+        status_output = self._safe_get_status()
 
         self.logger.debug("Received status output:\n%s", status_output)
 
@@ -506,6 +585,15 @@ class XIdleTime(Activity):
                     "Unable to parse configuration: {}".format(error),
                 ) from error
 
+    @staticmethod
+    def _get_session_method(method: str) -> Callable[[], List[XorgSession]]:
+        if method == "sockets":
+            return list_sessions_sockets
+        elif method == "logind":
+            return list_sessions_logind
+        else:
+            raise ValueError("Unknown session discovery method {}".format(method))
+
     def __init__(
         self,
         name: str,
@@ -516,93 +604,24 @@ class XIdleTime(Activity):
     ) -> None:
         Activity.__init__(self, name)
         self._timeout = timeout
-        if method == "sockets":
-            self._provide_sessions = self._list_sessions_sockets
-        elif method == "logind":
-            self._provide_sessions = self._list_sessions_logind
-        else:
-            raise ValueError("Unknown session discovery method {}".format(method))
+        self._provide_sessions: Callable[[], List[XorgSession]]
+        self._provide_sessions = self._get_session_method(method)
         self._ignore_process_re = ignore_process_re
         self._ignore_users_re = ignore_users_re
 
-    def _list_sessions_sockets(self) -> Sequence[Tuple[int, str]]:
-        """List running X sessions by iterating the X sockets.
-
-        This method assumes that X servers are run under the users using the
-        server.
-        """
-        sockets = glob.glob("/tmp/.X11-unix/X*")
-        self.logger.debug("Found sockets: %s", sockets)
-
-        results = []
-        for sock in sockets:
-            # determine the number of the X display
-            try:
-                display = int(sock[len("/tmp/.X11-unix/X") :])
-            except ValueError:
-                self.logger.warning(
-                    "Cannot parse display number from socket %s. Skipping.",
-                    sock,
-                    exc_info=True,
-                )
-                continue
-
-            # determine the user of the display
-            try:
-                user = pwd.getpwuid(os.stat(sock).st_uid).pw_name
-            except (FileNotFoundError, KeyError):
-                self.logger.warning(
-                    "Cannot get the owning user from socket %s. Skipping.",
-                    sock,
-                    exc_info=True,
-                )
-                continue
-
-            results.append((display, user))
-
-        return results
-
-    def _list_sessions_logind(self) -> Sequence[Tuple[int, str]]:
-        """List running X sessions using logind.
-
-        This method assumes that a ``Display`` variable is set in the logind
-        sessions.
-        """
-        results = []
-        for session_id, properties in list_logind_sessions():
-            if "Name" in properties and "Display" in properties:
-                try:
-                    results.append(
-                        (
-                            int(properties["Display"].replace(":", "")),
-                            str(properties["Name"]),
-                        )
-                    )
-                except ValueError:
-                    self.logger.warning(
-                        "Unable to parse display from session properties %s",
-                        properties,
-                        exc_info=True,
-                    )
-            else:
-                self.logger.debug(
-                    "Skipping session %s because it does not contain "
-                    "a user name and a display",
-                    session_id,
-                )
-        return results
-
-    def _is_skip_process_running(self, user: str) -> bool:
+    @staticmethod
+    def _get_user_processes(user: str) -> List[psutil.Process]:
         user_processes = []
         for process in psutil.process_iter():
-            try:
+            with suppress(
+                psutil.NoSuchProcess, psutil.ZombieProcess, psutil.AccessDenied
+            ):
                 if process.username() == user:
                     user_processes.append(process.name())
-            except (psutil.NoSuchProcess, psutil.ZombieProcess, psutil.AccessDenied):
-                # ignore processes which have disappeared etc.
-                pass
+        return user_processes
 
-        for process in user_processes:
+    def _is_skip_process_running(self, user: str) -> bool:
+        for process in self._get_user_processes(user):
             if self._ignore_process_re.match(process) is not None:
                 self.logger.debug(
                     "Process %s with pid %s matches the ignore regex '%s'."
@@ -615,46 +634,53 @@ class XIdleTime(Activity):
 
         return False
 
+    def _safe_provide_sessions(self) -> List[XorgSession]:
+        try:
+            return self._provide_sessions()
+        except LogindDBusException as error:
+            raise TemporaryCheckError(error) from error
+
+    def _get_idle_time(self, session: XorgSession) -> float:
+        env = copy.deepcopy(os.environ)
+        env["DISPLAY"] = ":{}".format(session.display)
+        env["XAUTHORITY"] = str(Path("~" + session.user).expanduser() / ".Xauthority")
+
+        try:
+            idle_time_output = subprocess.check_output(  # noqa: S603, S607
+                ["sudo", "-u", session.user, "xprintidle"], env=env
+            )
+            return float(idle_time_output.strip()) / 1000.0
+        except FileNotFoundError as error:
+            raise SevereCheckError("sudo executable not found") from error
+        except (subprocess.CalledProcessError, ValueError) as error:
+            self.logger.warning(
+                "Unable to determine the idle time for display %s.",
+                session.display,
+                exc_info=True,
+            )
+            raise TemporaryCheckError("Unable to call xprintidle") from error
+
     def check(self) -> Optional[str]:
-        for display, user in self._provide_sessions():
-            self.logger.info("Checking display %s of user %s", display, user)
+        for session in self._safe_provide_sessions():
+            self.logger.info("Checking session %s", session)
 
             # check whether this users should be ignored completely
-            if self._ignore_users_re.match(user) is not None:
-                self.logger.debug("Skipping user '%s' due to request", user)
+            if self._ignore_users_re.match(session.user) is not None:
+                self.logger.debug("Skipping user '%s' due to request", session.user)
                 continue
 
             # check whether any of the running processes of this user matches
             # the ignore regular expression. In that case we skip idletime
             # checking because we assume the user has a process running that
             # inevitably tampers with the idle time.
-            if self._is_skip_process_running(user):
+            if self._is_skip_process_running(session.user):
                 continue
 
-            # prepare the environment for the xprintidle call
-            env = copy.deepcopy(os.environ)
-            env["DISPLAY"] = ":{}".format(display)
-            env["XAUTHORITY"] = os.path.join(
-                os.path.expanduser("~" + user), ".Xauthority"
-            )
-
-            try:
-                idle_time_output = subprocess.check_output(  # noqa: S603, S607
-                    ["sudo", "-u", user, "xprintidle"], env=env
-                )
-                idle_time = float(idle_time_output.strip()) / 1000.0
-            except (subprocess.CalledProcessError, ValueError) as error:
-                self.logger.warning(
-                    "Unable to determine the idle time for display %s.",
-                    display,
-                    exc_info=True,
-                )
-                raise TemporaryCheckError(error) from error
-
+            idle_time = self._get_idle_time(session)
             self.logger.debug(
                 "Idle time for display %s of user %s is %s seconds.",
-                display,
-                user,
+                session.display,
+                session.user,
                 idle_time,
             )
 
@@ -662,7 +688,7 @@ class XIdleTime(Activity):
                 return (
                     "X session {} of user {} "
                     "has idle time {} < threshold {}".format(
-                        display, user, idle_time, self._timeout
+                        session.display, session.user, idle_time, self._timeout
                     )
                 )
 
@@ -677,7 +703,9 @@ class LogindSessionsIdle(Activity):
 
     @classmethod
     def create(
-        cls, name: str, config: configparser.SectionProxy,
+        cls,
+        name: str,
+        config: configparser.SectionProxy,
     ) -> "LogindSessionsIdle":
         types = config.get("types", fallback="tty,x11,wayland").split(",")
         types = [t.strip() for t in types]
@@ -690,8 +718,15 @@ class LogindSessionsIdle(Activity):
         self._types = types
         self._states = states
 
+    @staticmethod
+    def _list_logind_sessions() -> Iterable[Tuple[str, dict]]:
+        try:
+            return list_logind_sessions()
+        except LogindDBusException as error:
+            raise TemporaryCheckError(error) from error
+
     def check(self) -> Optional[str]:
-        for session_id, properties in list_logind_sessions():
+        for session_id, properties in self._list_logind_sessions():
             self.logger.debug("Session %s properties: %s", session_id, properties)
 
             if properties["Type"] not in self._types:
@@ -712,7 +747,7 @@ class LogindSessionsIdle(Activity):
 
 
 class XPath(XPathMixin, Activity):
-    def __init__(self, name: str, **kwargs) -> None:
+    def __init__(self, name: str, **kwargs: Any) -> None:
         Activity.__init__(self, name)
         XPathMixin.__init__(self, **kwargs)
 
@@ -721,3 +756,141 @@ class XPath(XPathMixin, Activity):
             return "XPath matches for url " + self._url
         else:
             return None
+
+
+class JsonPath(NetworkMixin, Activity):
+    """Requests a URL and evaluates whether a JSONPath expression matches."""
+
+    @classmethod
+    def collect_init_args(cls, config: configparser.SectionProxy) -> Dict[str, Any]:
+        from jsonpath_ng.ext import parse
+
+        try:
+            args = NetworkMixin.collect_init_args(config)
+            args["jsonpath"] = parse(config["jsonpath"])
+            return args
+        except KeyError as error:
+            raise ConfigurationError(f"Property jsonpath is missing") from error
+        except Exception as error:
+            raise ConfigurationError(f"JSONPath error {str(error)}") from error
+
+    def __init__(self, name: str, jsonpath: "JSONPath", **kwargs: Any) -> None:
+        Activity.__init__(self, name)
+        NetworkMixin.__init__(self, accept="application/json", **kwargs)
+        self._jsonpath = jsonpath
+
+    def check(self) -> Optional[str]:
+        import requests
+        import requests.exceptions
+
+        try:
+            reply = self.request().json()
+            matched = self._jsonpath.find(reply)
+            if matched:
+                # shorten to avoid excessive logging output
+                return f"JSONPath {self._jsonpath} found elements " + shorten(
+                    str(matched), 24
+                )
+            return None
+        except (json.JSONDecodeError, requests.exceptions.RequestException) as error:
+            raise TemporaryCheckError(error) from error
+
+
+class LastLogActivity(Activity):
+    @classmethod
+    def create(cls, name: str, config: configparser.SectionProxy) -> "LastLogActivity":
+        import pytz
+
+        try:
+            return cls(
+                name,
+                Path(config["log_file"]),
+                re.compile(config["pattern"]),
+                timedelta(minutes=config.getint("minutes", fallback=10)),
+                config.get("encoding", "ascii"),
+                pytz.timezone(config.get("timezone", "UTC")),  # type: ignore
+            )
+        except KeyError as error:
+            raise ConfigurationError(
+                "Missing config key {}".format(error),
+            ) from error
+        except re.error as error:
+            raise ConfigurationError(
+                "Regular expression is invalid: {}".format(error),
+            ) from error
+        except ValueError as error:
+            raise ConfigurationError(
+                "Unable to parse configuration: {}".format(error),
+            ) from error
+
+    def __init__(
+        self,
+        name: str,
+        log_file: Path,
+        pattern: Pattern,
+        delta: timedelta,
+        encoding: str,
+        default_timezone: timezone,
+    ) -> None:
+        if delta.total_seconds() < 0:
+            raise ValueError("Given delta must be positive")
+        if pattern.groups != 1:
+            raise ValueError("Given pattern must have exactly one capture group")
+        super().__init__(name=name)
+        self.log_file = log_file
+        self.pattern = pattern
+        self.delta = delta
+        self.encoding = encoding
+        self.default_timezone = default_timezone
+
+    def _safe_parse_date(self, match: str, now: datetime) -> datetime:
+        from dateutil.parser import parse
+        from dateutil.utils import default_tzinfo
+
+        try:
+            match_date = default_tzinfo(parse(match), self.default_timezone)
+            if match_date > now:
+                raise TemporaryCheckError(
+                    "Detected date {} is in the future".format(match_date)
+                )
+            return match_date
+        except ValueError as error:
+            raise TemporaryCheckError(
+                "Detected date {} cannot be parsed as a date".format(match)
+            ) from error
+        except OverflowError as error:
+            raise TemporaryCheckError(
+                "Detected date {} is out of the valid range".format(match)
+            ) from error
+
+    def _file_lines_reversed(self) -> Iterable[str]:
+        try:
+            # Probably not the most effective solution for large log files. Might need
+            # optimizations later on.
+            return reversed(
+                self.log_file.read_text(encoding=self.encoding).splitlines()
+            )
+        except IOError as error:
+            raise TemporaryCheckError(
+                "Cannot access log file {}".format(self.log_file)
+            ) from error
+
+    def check(self) -> Optional[str]:
+        lines = self._file_lines_reversed()
+
+        now = datetime.now(tz=timezone.utc)
+        for line in lines:
+            match = self.pattern.match(line)
+            if not match:
+                continue
+
+            match_date = self._safe_parse_date(match.group(1), now)
+
+            # Only check the first line (reverse order) that has a match, not all
+            if (now - match_date) < self.delta:
+                return "Log activity in {} at {}".format(self.log_file, match_date)
+            else:
+                return None
+
+        # No line matched at all
+        return None
