@@ -1,17 +1,19 @@
 import configparser
 from datetime import datetime, timedelta, timezone
 from io import BytesIO
+from pathlib import Path
 import subprocess
-from typing import Optional
+from typing import Any, Optional
 
+from . import ConfigurationError, TemporaryCheckError, Wakeup
 from .util import CommandMixin, NetworkMixin, XPathMixin
-from .. import ConfigurationError, TemporaryCheckError, Wakeup
+from ..util.subprocess import raise_severe_if_command_not_found
 
 
 class Calendar(NetworkMixin, Wakeup):
     """Uses an ical calendar to wake up on the next scheduled event."""
 
-    def __init__(self, name: str, **kwargs) -> None:
+    def __init__(self, name: str, **kwargs: Any) -> None:
         NetworkMixin.__init__(self, **kwargs)
         Wakeup.__init__(self, name)
 
@@ -44,26 +46,26 @@ class File(Wakeup):
     @classmethod
     def create(cls, name: str, config: configparser.SectionProxy) -> "File":
         try:
-            path = config["path"]
+            path = Path(config["path"])
             return cls(name, path)
         except KeyError as error:
             raise ConfigurationError("Missing option path") from error
 
-    def __init__(self, name: str, path: str) -> None:
+    def __init__(self, name: str, path: Path) -> None:
         Wakeup.__init__(self, name)
         self._path = path
 
     def check(self, timestamp: datetime) -> Optional[datetime]:
         try:
-            with open(self._path, "r") as time_file:
-                return datetime.fromtimestamp(
-                    float(time_file.readlines()[0].strip()), timezone.utc
-                )
+            first_line = self._path.read_text().splitlines()[0]
+            return datetime.fromtimestamp(float(first_line.strip()), timezone.utc)
         except FileNotFoundError:
             # this is ok
             return None
         except (ValueError, IOError) as error:
-            raise TemporaryCheckError(error) from error
+            raise TemporaryCheckError(
+                "Next wakeup time cannot be read despite a file being present"
+            ) from error
 
 
 class Command(CommandMixin, Wakeup):
@@ -80,7 +82,8 @@ class Command(CommandMixin, Wakeup):
     def check(self, timestamp: datetime) -> Optional[datetime]:
         try:
             output = subprocess.check_output(
-                self._command, shell=True,  # noqa: S602
+                self._command,
+                shell=True,  # noqa: S602
             ).splitlines()[0]
             self.logger.debug(
                 "Command %s succeeded with output %s", self._command, output
@@ -90,8 +93,15 @@ class Command(CommandMixin, Wakeup):
             else:
                 return None
 
-        except (subprocess.CalledProcessError, ValueError) as error:
-            raise TemporaryCheckError(error) from error
+        except subprocess.CalledProcessError as error:
+            raise_severe_if_command_not_found(error)
+            raise TemporaryCheckError(
+                "Unable to call the configured command"
+            ) from error
+        except ValueError as error:
+            raise TemporaryCheckError(
+                "Return value cannot be interpreted as a timestamp"
+            ) from error
 
 
 class Periodic(Wakeup):
@@ -123,7 +133,7 @@ class XPath(XPathMixin, Wakeup):
     The matched results are expected to represent timestamps in seconds UTC.
     """
 
-    def __init__(self, name: str, **kwargs) -> None:
+    def __init__(self, name: str, **kwargs: Any) -> None:
         Wakeup.__init__(self, name)
         XPathMixin.__init__(self, **kwargs)
 
@@ -166,7 +176,7 @@ class XPathDelta(XPath):
         except ValueError as error:
             raise ConfigurationError(str(error))
 
-    def __init__(self, name: str, unit: str, **kwargs) -> None:
+    def __init__(self, name: str, unit: str, **kwargs: Any) -> None:
         if unit not in self.UNITS:
             raise ValueError("Unsupported unit")
         XPath.__init__(self, name, **kwargs)
