@@ -3,21 +3,23 @@ import configparser
 from datetime import datetime, timedelta, timezone
 import logging
 import subprocess
+from typing import Any, Optional
 
 import dateutil.parser
 import pytest
+from pytest_mock import MockFixture
 
 import autosuspend
 
 
 class TestExecuteSuspend:
-    def test_smoke(self, mocker) -> None:
+    def test_smoke(self, mocker: MockFixture) -> None:
         mock = mocker.patch("subprocess.check_call")
         command = ["foo", "bar"]
         autosuspend.execute_suspend(command, None)
         mock.assert_called_once_with(command, shell=True)
 
-    def test_call_exception(self, mocker) -> None:
+    def test_call_exception(self, mocker: MockFixture) -> None:
         mock = mocker.patch("subprocess.check_call")
         command = ["foo", "bar"]
         mock.side_effect = subprocess.CalledProcessError(2, command)
@@ -31,7 +33,7 @@ class TestExecuteSuspend:
 
 
 class TestScheduleWakeup:
-    def test_smoke(self, mocker) -> None:
+    def test_smoke(self, mocker: MockFixture) -> None:
         mock = mocker.patch("subprocess.check_call")
         dt = datetime.fromtimestamp(1525270801, timezone(timedelta(hours=4)))
         autosuspend.schedule_wakeup("echo {timestamp:.0f} {iso}", dt)
@@ -39,7 +41,7 @@ class TestScheduleWakeup:
             "echo 1525270801 2018-05-02T18:20:01+04:00", shell=True
         )
 
-    def test_call_exception(self, mocker) -> None:
+    def test_call_exception(self, mocker: MockFixture) -> None:
         mock = mocker.patch("subprocess.check_call")
         mock.side_effect = subprocess.CalledProcessError(2, "foo")
 
@@ -52,21 +54,21 @@ class TestScheduleWakeup:
 
 
 class TestConfigureLogging:
-    def test_debug(self, mocker) -> None:
+    def test_debug(self, mocker: MockFixture) -> None:
         mock = mocker.patch("logging.basicConfig")
 
         autosuspend.configure_logging(None, True)
 
         mock.assert_called_once_with(level=logging.DEBUG)
 
-    def test_standard(self, mocker) -> None:
+    def test_standard(self, mocker: MockFixture) -> None:
         mock = mocker.patch("logging.basicConfig")
 
         autosuspend.configure_logging(None, False)
 
         mock.assert_called_once_with(level=logging.WARNING)
 
-    def test_file(self, mocker) -> None:
+    def test_file(self, mocker: MockFixture) -> None:
         mock = mocker.patch("logging.config.fileConfig")
 
         # anything that is not a boolean is treated like a file
@@ -74,7 +76,7 @@ class TestConfigureLogging:
 
         mock.assert_called_once_with(42)
 
-    def test_file_fallback(self, mocker) -> None:
+    def test_file_fallback(self, mocker: MockFixture) -> None:
         mock = mocker.patch("logging.config.fileConfig", side_effect=RuntimeError())
         mock_basic = mocker.patch("logging.basicConfig")
 
@@ -86,7 +88,7 @@ class TestConfigureLogging:
 
 
 class TestSetUpChecks:
-    def test_smoke(self, mocker) -> None:
+    def test_smoke(self, mocker: MockFixture) -> None:
         mock_class = mocker.patch("autosuspend.checks.activity.Mpd")
         mock_class.create.return_value = mocker.MagicMock(
             spec=autosuspend.checks.Activity
@@ -107,7 +109,7 @@ class TestSetUpChecks:
 
         mock_class.create.assert_called_once_with("Foo", parser["check.Foo"])
 
-    def test_external_class(self, mocker) -> None:
+    def test_external_class(self, mocker: MockFixture) -> None:
         mock_class = mocker.patch("os.path.TestCheck", create=True)
         mock_class.create.return_value = mocker.MagicMock(
             spec=autosuspend.checks.Activity
@@ -127,7 +129,7 @@ class TestSetUpChecks:
 
         mock_class.create.assert_called_once_with("Foo", parser["check.Foo"])
 
-    def test_not_enabled(self, mocker) -> None:
+    def test_not_enabled(self, mocker: MockFixture) -> None:
         mock_class = mocker.patch("autosuspend.checks.activity.Mpd")
         mock_class.create.return_value = mocker.MagicMock(spec=autosuspend.Activity)
 
@@ -141,7 +143,10 @@ class TestSetUpChecks:
         )
 
         assert not autosuspend.set_up_checks(
-            parser, "check", "activity", autosuspend.Activity,  # type: ignore
+            parser,
+            "check",
+            "activity",
+            autosuspend.Activity,  # type: ignore
         )
 
         with pytest.raises(autosuspend.ConfigurationError):
@@ -153,7 +158,7 @@ class TestSetUpChecks:
                 error_none=True,
             )
 
-    def test_not_enabled_continues_with_next(self, mocker) -> None:
+    def test_not_enabled_continues_with_next(self, mocker: MockFixture) -> None:
         mock_mpd = mocker.patch("autosuspend.checks.activity.Mpd")
         mock_mpd.create.return_value = mocker.MagicMock(spec=autosuspend.Activity)
         mock_xidletime = mocker.patch("autosuspend.checks.activity.XIdleTime")
@@ -174,13 +179,16 @@ class TestSetUpChecks:
         assert (
             len(
                 autosuspend.set_up_checks(
-                    parser, "check", "activity", autosuspend.Activity,  # type: ignore
+                    parser,
+                    "check",
+                    "activity",
+                    autosuspend.Activity,  # type: ignore
                 )
             )
             == 1
         )
 
-    def test_no_such_class(self, mocker) -> None:
+    def test_no_such_class(self, mocker: MockFixture) -> None:
         parser = configparser.ConfigParser()
         parser.read_string(
             """
@@ -194,7 +202,7 @@ class TestSetUpChecks:
                 parser, "check", "activity", autosuspend.Activity  # type: ignore
             )
 
-    def test_not_a_check(self, mocker) -> None:
+    def test_not_a_check(self, mocker: MockFixture) -> None:
         mock_class = mocker.patch("autosuspend.checks.activity.Mpd")
         mock_class.create.return_value = mocker.MagicMock()
 
@@ -214,7 +222,7 @@ class TestSetUpChecks:
 
         mock_class.create.assert_called_once_with("Foo", parser["check.Foo"])
 
-    def test_passwords_redacted(self, mocker, caplog) -> None:
+    def test_passwords_redacted(self, mocker: MockFixture, caplog: Any) -> None:
         mock_class = mocker.patch("autosuspend.checks.activity.Mpd")
         mock_class.create.return_value = mocker.MagicMock(
             spec=autosuspend.checks.Activity
@@ -239,10 +247,10 @@ class TestSetUpChecks:
 
 
 class TestExecuteChecks:
-    def test_no_checks(self, mocker) -> None:
+    def test_no_checks(self, mocker: MockFixture) -> None:
         assert autosuspend.execute_checks([], False, mocker.MagicMock()) is False
 
-    def test_matches(self, mocker) -> None:
+    def test_matches(self, mocker: MockFixture) -> None:
         matching_check = mocker.MagicMock(spec=autosuspend.Activity)
         matching_check.name = "foo"
         matching_check.check.return_value = "matches"
@@ -252,7 +260,7 @@ class TestExecuteChecks:
         )
         matching_check.check.assert_called_once_with()
 
-    def test_only_first_called(self, mocker) -> None:
+    def test_only_first_called(self, mocker: MockFixture) -> None:
         matching_check = mocker.MagicMock(spec=autosuspend.Activity)
         matching_check.name = "foo"
         matching_check.check.return_value = "matches"
@@ -269,7 +277,7 @@ class TestExecuteChecks:
         matching_check.check.assert_called_once_with()
         second_check.check.assert_not_called()
 
-    def test_all_called(self, mocker) -> None:
+    def test_all_called(self, mocker: MockFixture) -> None:
         matching_check = mocker.MagicMock(spec=autosuspend.Activity)
         matching_check.name = "foo"
         matching_check.check.return_value = "matches"
@@ -286,7 +294,7 @@ class TestExecuteChecks:
         matching_check.check.assert_called_once_with()
         second_check.check.assert_called_once_with()
 
-    def test_ignore_temporary_errors(self, mocker) -> None:
+    def test_ignore_temporary_errors(self, mocker: MockFixture) -> None:
         matching_check = mocker.MagicMock(spec=autosuspend.Activity)
         matching_check.name = "foo"
         matching_check.check.side_effect = autosuspend.TemporaryCheckError()
@@ -305,7 +313,7 @@ class TestExecuteChecks:
 
 
 class TestExecuteWakeups:
-    def test_no_wakeups(self, mocker) -> None:
+    def test_no_wakeups(self, mocker: MockFixture) -> None:
         assert (
             autosuspend.execute_wakeups(
                 [], datetime.now(timezone.utc), mocker.MagicMock()
@@ -313,7 +321,7 @@ class TestExecuteWakeups:
             is None
         )
 
-    def test_all_none(self, mocker) -> None:
+    def test_all_none(self, mocker: MockFixture) -> None:
         wakeup = mocker.MagicMock(spec=autosuspend.Wakeup)
         wakeup.check.return_value = None
         assert (
@@ -324,9 +332,12 @@ class TestExecuteWakeups:
         )
 
     @pytest.mark.parametrize(
-        "illegal", [None, dateutil.parser.parse("20040605T090000Z")],
+        "illegal",
+        [None, dateutil.parser.parse("20040605T090000Z")],
     )
-    def test_skips_none_outdated_and_continues(self, mocker, illegal) -> None:
+    def test_skips_none_outdated_and_continues(
+        self, mocker: MockFixture, illegal: Optional[datetime]
+    ) -> None:
         wakeup_none = mocker.MagicMock(spec=autosuspend.Wakeup)
         wakeup_none.check.return_value = illegal
         now = dateutil.parser.parse("20040705T090000Z")
@@ -335,13 +346,15 @@ class TestExecuteWakeups:
         wakeup_real.check.return_value = wake_up_at
         assert (
             autosuspend.execute_wakeups(
-                [wakeup_none, wakeup_real], now, mocker.MagicMock(),
+                [wakeup_none, wakeup_real],
+                now,
+                mocker.MagicMock(),
             )
             == wake_up_at
         )
         assert wakeup_none.check.called
 
-    def test_basic_return(self, mocker) -> None:
+    def test_basic_return(self, mocker: MockFixture) -> None:
         wakeup = mocker.MagicMock(spec=autosuspend.Wakeup)
         now = datetime.now(timezone.utc)
         wakeup_time = now + timedelta(seconds=10)
@@ -351,7 +364,7 @@ class TestExecuteWakeups:
             == wakeup_time
         )
 
-    def test_soonest_taken(self, mocker) -> None:
+    def test_soonest_taken(self, mocker: MockFixture) -> None:
         reference = datetime.now(timezone.utc)
         wakeup = mocker.MagicMock(spec=autosuspend.Wakeup)
         wakeup.check.return_value = reference + timedelta(seconds=20)
@@ -368,7 +381,7 @@ class TestExecuteWakeups:
             == earlier
         )
 
-    def test_ignore_temporary_errors(self, mocker) -> None:
+    def test_ignore_temporary_errors(self, mocker: MockFixture) -> None:
         now = datetime.now(timezone.utc)
 
         wakeup = mocker.MagicMock(spec=autosuspend.Wakeup)
@@ -381,7 +394,7 @@ class TestExecuteWakeups:
             [wakeup, wakeup_error, wakeup_earlier], now, mocker.MagicMock()
         ) == now + timedelta(seconds=10)
 
-    def test_ignore_too_early(self, mocker) -> None:
+    def test_ignore_too_early(self, mocker: MockFixture) -> None:
         now = datetime.now(timezone.utc)
         wakeup = mocker.MagicMock(spec=autosuspend.Wakeup)
         wakeup.check.return_value = now
@@ -395,7 +408,7 @@ class TestExecuteWakeups:
 
 
 class TestNotifySuspend:
-    def test_date(self, mocker) -> None:
+    def test_date(self, mocker: MockFixture) -> None:
         mock = mocker.patch("subprocess.check_call")
         dt = datetime.fromtimestamp(1525270801, timezone(timedelta(hours=4)))
         autosuspend.notify_suspend("echo {timestamp:.0f} {iso}", "not this", dt)
@@ -403,23 +416,23 @@ class TestNotifySuspend:
             "echo 1525270801 2018-05-02T18:20:01+04:00", shell=True
         )
 
-    def test_date_no_command(self, mocker) -> None:
+    def test_date_no_command(self, mocker: MockFixture) -> None:
         mock = mocker.patch("subprocess.check_call")
         dt = datetime.fromtimestamp(1525270801, timezone(timedelta(hours=4)))
         autosuspend.notify_suspend(None, "not this", dt)
         mock.assert_not_called()
 
-    def test_no_date(self, mocker) -> None:
+    def test_no_date(self, mocker: MockFixture) -> None:
         mock = mocker.patch("subprocess.check_call")
         autosuspend.notify_suspend("echo {timestamp:.0f} {iso}", "echo nothing", None)
         mock.assert_called_once_with("echo nothing", shell=True)
 
-    def test_no_date_no_command(self, mocker) -> None:
+    def test_no_date_no_command(self, mocker: MockFixture) -> None:
         mock = mocker.patch("subprocess.check_call")
         autosuspend.notify_suspend("echo {timestamp:.0f} {iso}", None, None)
         mock.assert_not_called()
 
-    def test_ignore_execution_errors(self, mocker, caplog) -> None:
+    def test_ignore_execution_errors(self, mocker: MockFixture, caplog: Any) -> None:
         mock = mocker.patch("subprocess.check_call")
         mock.side_effect = subprocess.CalledProcessError(2, "cmd")
         dt = datetime.fromtimestamp(1525270801, timezone(timedelta(hours=4)))
@@ -428,14 +441,14 @@ class TestNotifySuspend:
             assert "Unable to execute" in caplog.text
             assert mock.called
 
-    def test_info_no_command(self, caplog) -> None:
+    def test_info_no_command(self, caplog: Any) -> None:
         with caplog.at_level(logging.INFO):
             autosuspend.notify_suspend(None, None, datetime.now())
             assert "suitable" in caplog.text
 
 
 class TestConfigureProcessor:
-    def test_minimal_config(self, mocker) -> None:
+    def test_minimal_config(self, mocker: MockFixture) -> None:
         parser = configparser.ConfigParser()
         parser.read_string(
             """
@@ -453,7 +466,7 @@ class TestConfigureProcessor:
         assert processor._all_activities
 
 
-def test_notify_and_suspend(mocker) -> None:
+def test_notify_and_suspend(mocker: MockFixture) -> None:
     mock = mocker.patch("subprocess.check_call")
     dt = datetime.fromtimestamp(1525270801, timezone(timedelta(hours=4)))
     autosuspend.notify_and_suspend(
@@ -469,52 +482,54 @@ def test_notify_and_suspend(mocker) -> None:
 
 class _StubCheck(autosuspend.Activity):
     @classmethod
-    def create(cls, name, config):
-        pass
+    def create(cls, name: str, config: configparser.SectionProxy) -> "_StubCheck":
+        raise NotImplementedError()
 
-    def __init__(self, name, match):
+    def __init__(self, name: str, match: Optional[str]) -> None:
         autosuspend.Activity.__init__(self, name)
         self.match = match
 
-    def check(self):
+    def check(self) -> Optional[str]:
         return self.match
 
 
-@pytest.fixture
-def sleep_fn():
-    class Func:
-        def __init__(self):
-            self.called = False
-            self.call_arg = None
+class SleepFn:
+    def __init__(self) -> None:
+        self.called = False
+        self.call_arg: Optional[float] = None
 
-        def reset(self):
-            self.called = False
-            self.call_arg = None
+    def reset(self) -> None:
+        self.called = False
+        self.call_arg = None
 
-        def __call__(self, arg):
-            self.called = True
-            self.call_arg = arg
-
-    return Func()
+    def __call__(self, arg: float) -> None:
+        self.called = True
+        self.call_arg = arg
 
 
-@pytest.fixture
-def wakeup_fn():
-    class Func:
-        def __init__(self):
-            self.call_arg = None
+@pytest.fixture()
+def sleep_fn() -> SleepFn:
+    return SleepFn()
 
-        def reset(self):
-            self.call_arg = None
 
-        def __call__(self, arg):
-            self.call_arg = arg
+class WakeupFn:
+    def __init__(self) -> None:
+        self.call_arg: Optional[datetime] = None
 
-    return Func()
+    def reset(self) -> None:
+        self.call_arg = None
+
+    def __call__(self, arg: datetime) -> None:
+        self.call_arg = arg
+
+
+@pytest.fixture()
+def wakeup_fn() -> WakeupFn:
+    return WakeupFn()
 
 
 class TestProcessor:
-    def test_smoke(self, sleep_fn, wakeup_fn) -> None:
+    def test_smoke(self, sleep_fn: SleepFn, wakeup_fn: WakeupFn) -> None:
         processor = autosuspend.Processor(
             [_StubCheck("stub", None)], [], 2, 0, 0, sleep_fn, wakeup_fn, False
         )
@@ -544,7 +559,9 @@ class TestProcessor:
 
         assert wakeup_fn.call_arg is None
 
-    def test_just_woke_up_handling(self, sleep_fn, wakeup_fn) -> None:
+    def test_just_woke_up_handling(
+        self, sleep_fn: SleepFn, wakeup_fn: WakeupFn
+    ) -> None:
         processor = autosuspend.Processor(
             [_StubCheck("stub", None)], [], 2, 0, 0, sleep_fn, wakeup_fn, False
         )
@@ -568,7 +585,9 @@ class TestProcessor:
 
         assert wakeup_fn.call_arg is None
 
-    def test_wakeup_blocks_sleep(self, mocker, sleep_fn, wakeup_fn) -> None:
+    def test_wakeup_blocks_sleep(
+        self, mocker: MockFixture, sleep_fn: SleepFn, wakeup_fn: WakeupFn
+    ) -> None:
         start = datetime.now(timezone.utc)
         wakeup = mocker.MagicMock(spec=autosuspend.Wakeup)
         wakeup.check.return_value = start + timedelta(seconds=6)
@@ -584,7 +603,10 @@ class TestProcessor:
         assert wakeup_fn.call_arg is None
 
     def test_wakeup_exact_hit_does_not_block(
-        self, mocker, sleep_fn, wakeup_fn,
+        self,
+        mocker: MockFixture,
+        sleep_fn: SleepFn,
+        wakeup_fn: WakeupFn,
     ) -> None:
         start = datetime.now(timezone.utc)
         wakeup = mocker.MagicMock(spec=autosuspend.Wakeup)
@@ -600,7 +622,9 @@ class TestProcessor:
         assert sleep_fn.called
         assert wakeup_fn.call_arg is not None
 
-    def test_wakeup_scheduled(self, mocker, sleep_fn, wakeup_fn) -> None:
+    def test_wakeup_scheduled(
+        self, mocker: MockFixture, sleep_fn: SleepFn, wakeup_fn: WakeupFn
+    ) -> None:
         start = datetime.now(timezone.utc)
         wakeup = mocker.MagicMock(spec=autosuspend.Wakeup)
         wakeup.check.return_value = start + timedelta(seconds=25)
@@ -623,7 +647,9 @@ class TestProcessor:
         processor.iteration(start + timedelta(seconds=25), False)
         assert wakeup_fn.call_arg is None
 
-    def test_wakeup_delta_blocks(self, mocker, sleep_fn, wakeup_fn) -> None:
+    def test_wakeup_delta_blocks(
+        self, mocker: MockFixture, sleep_fn: SleepFn, wakeup_fn: WakeupFn
+    ) -> None:
         start = datetime.now(timezone.utc)
         wakeup = mocker.MagicMock(spec=autosuspend.Wakeup)
         wakeup.check.return_value = start + timedelta(seconds=25)
@@ -637,7 +663,9 @@ class TestProcessor:
         processor.iteration(start + timedelta(seconds=3), False)
         assert not sleep_fn.called
 
-    def test_wakeup_delta_applied(self, mocker, sleep_fn, wakeup_fn) -> None:
+    def test_wakeup_delta_applied(
+        self, mocker: MockFixture, sleep_fn: SleepFn, wakeup_fn: WakeupFn
+    ) -> None:
         start = datetime.now(timezone.utc)
         wakeup = mocker.MagicMock(spec=autosuspend.Wakeup)
         wakeup.check.return_value = start + timedelta(seconds=25)
