@@ -1,4 +1,5 @@
 from datetime import timedelta
+from pathlib import Path
 
 from dateutil import parser
 from dateutil.tz import tzlocal
@@ -16,7 +17,7 @@ class TestCalendarEvent:
 
 
 class TestListCalendarEvents:
-    def test_simple_recurring(self, datadir) -> None:
+    def test_simple_recurring(self, datadir: Path) -> None:
         """Tests for basic recurrence.
 
         Events are collected with the same DST setting as their original
@@ -56,7 +57,7 @@ class TestListCalendarEvents:
             assert expected_start_times == [e.start for e in events]
             assert expected_end_times == [e.end for e in events]
 
-    def test_recurrence_different_dst(self, datadir) -> None:
+    def test_recurrence_different_dst(self, datadir: Path) -> None:
         with (datadir / "simple-recurring.ics").open("rb") as f:
             start = parser.parse("2018-11-19 04:00:00 UTC")
             end = start + timedelta(weeks=2)
@@ -77,7 +78,7 @@ class TestListCalendarEvents:
 
             assert expected_start_times == [e.start for e in events]
 
-    def test_all_day_events(self, datadir) -> None:
+    def test_all_day_events(self, datadir: Path) -> None:
         with (datadir / "all-day-events.ics").open("rb") as f:
             start = parser.parse("2018-06-11 02:00:00 UTC")
             end = start + timedelta(weeks=1)
@@ -87,7 +88,7 @@ class TestListCalendarEvents:
             expected_summaries = ["start", "between", "end"]
             assert [e.summary for e in events] == expected_summaries
 
-    def test_normal_events(self, datadir) -> None:
+    def test_normal_events(self, datadir: Path) -> None:
         with (datadir / "normal-events-corner-cases.ics").open("rb") as f:
             start = parser.parse("2018-06-04 00:00:00 +0200")
             end = start + timedelta(weeks=1)
@@ -128,7 +129,7 @@ class TestListCalendarEvents:
 
             assert [(e.summary, e.start, e.end) for e in events] == expected
 
-    def test_floating_time(self, datadir) -> None:
+    def test_floating_time(self, datadir: Path) -> None:
         with (datadir / "floating.ics").open("rb") as f:
             start = parser.parse("2018-06-09 00:00:00 +0200")
             end = start + timedelta(weeks=1)
@@ -166,7 +167,7 @@ class TestListCalendarEvents:
 
             assert [(e.summary, e.start, e.end) for e in events] == expected
 
-    def test_floating_time_other_dst(self, datadir) -> None:
+    def test_floating_time_other_dst(self, datadir: Path) -> None:
         with (datadir / "floating.ics").open("rb") as f:
             start = parser.parse("2018-12-09 00:00:00 +0200")
             end = start + timedelta(weeks=1)
@@ -214,7 +215,7 @@ class TestListCalendarEvents:
 
             assert [(e.summary, e.start, e.end) for e in events] == expected
 
-    def test_exclusions(self, datadir) -> None:
+    def test_exclusions(self, datadir: Path) -> None:
         with (datadir / "exclusions.ics").open("rb") as f:
             start = parser.parse("2018-06-09 04:00:00 UTC")
             end = start + timedelta(weeks=2)
@@ -231,7 +232,7 @@ class TestListCalendarEvents:
 
             assert expected_start_times == [e.start for e in events]
 
-    def test_reucrring_single_changes(self, datadir) -> None:
+    def test_reucrring_single_changes(self, datadir: Path) -> None:
         with (datadir / "single-change.ics").open("rb") as f:
             start = parser.parse("2018-06-11 00:00:00 UTC")
             end = start + timedelta(weeks=1)
@@ -249,7 +250,7 @@ class TestListCalendarEvents:
 
             assert expected_start_times == [e.start for e in events]
 
-    def test_reucrring_change_dst(self, datadir) -> None:
+    def test_reucrring_change_dst(self, datadir: Path) -> None:
         with (datadir / "recurring-change-dst.ics").open("rb") as f:
             start = parser.parse("2018-12-10 00:00:00 UTC")
             end = start + timedelta(weeks=1)
@@ -266,7 +267,7 @@ class TestListCalendarEvents:
 
             assert expected_start_times == [e.start for e in events]
 
-    def test_recurring_start_and_end_inclusive(self, datadir) -> None:
+    def test_recurring_start_and_end_inclusive(self, datadir: Path) -> None:
         with (datadir / "issue-41.ics").open("rb") as f:
             start = parser.parse("2018-06-26 15:13:51 UTC")
             end = start + timedelta(weeks=1)
@@ -285,7 +286,7 @@ class TestListCalendarEvents:
 
             assert expected_start_times == [e.start for e in events]
 
-    def test_single_start_end_inclusive(self, datadir) -> None:
+    def test_single_start_end_inclusive(self, datadir: Path) -> None:
         with (datadir / "old-event.ics").open("rb") as f:
             start = parser.parse("2004-06-05 11:15:00 UTC")
             end = start + timedelta(hours=1)
@@ -297,7 +298,7 @@ class TestListCalendarEvents:
 
             assert expected_start_times == [e.start for e in events]
 
-    def test_single_all_day_start_end_inclusive(self, datadir) -> None:
+    def test_single_all_day_start_end_inclusive(self, datadir: Path) -> None:
         with (datadir / "all-day-starts.ics").open("rb") as f:
             start = parser.parse("2018-06-25 10:00:00 UTC")
             end = start + timedelta(hours=2)
@@ -315,7 +316,7 @@ class TestListCalendarEvents:
 
             assert expected_end_times == [e.end for e in events]
 
-    def test_longer_single_all_day_start_end_inclusive(self, datadir) -> None:
+    def test_longer_single_all_day_start_end_inclusive(self, datadir: Path) -> None:
         with (datadir / "all-day-starts.ics").open("rb") as f:
             start = parser.parse("2018-06-29 10:00:00 UTC")
             end = start + timedelta(hours=2)
@@ -327,7 +328,7 @@ class TestListCalendarEvents:
 
             assert expected_start_times == [e.start for e in events]
 
-    def test_recurring_all_day_start_end_inclusive(self, datadir) -> None:
+    def test_recurring_all_day_start_end_inclusive(self, datadir: Path) -> None:
         with (datadir / "all-day-recurring.ics").open("rb") as f:
             start = parser.parse("2018-06-29 10:00:00 UTC")
             end = start + timedelta(hours=2)
@@ -345,7 +346,7 @@ class TestListCalendarEvents:
 
             assert expected_end_times == [e.end for e in events]
 
-    def test_recurring_all_day_start_in_between(self, datadir) -> None:
+    def test_recurring_all_day_start_in_between(self, datadir: Path) -> None:
         with (datadir / "all-day-recurring.ics").open("rb") as f:
             start = parser.parse("2018-06-29 00:00:00 UTC")
             end = start + timedelta(days=1)
@@ -358,7 +359,7 @@ class TestListCalendarEvents:
 
             assert expected_start_times == [e.start for e in events]
 
-    def test_recurring_all_day_exclusions(self, datadir) -> None:
+    def test_recurring_all_day_exclusions(self, datadir: Path) -> None:
         with (datadir / "all-day-recurring-exclusions.ics").open("rb") as f:
             start = parser.parse("2018-06-27 00:00:00 UTC")
             end = start + timedelta(days=4)
@@ -373,7 +374,7 @@ class TestListCalendarEvents:
 
             assert expected_start_times == [e.start for e in events]
 
-    def test_recurring_all_day_exclusions_end(self, datadir) -> None:
+    def test_recurring_all_day_exclusions_end(self, datadir: Path) -> None:
         with (datadir / "all-day-recurring-exclusions.ics").open("rb") as f:
             start = parser.parse("2018-06-26 00:00:00 UTC")
             end = start + timedelta(days=4)
